@@ -30,6 +30,9 @@
 #include "lqt_codecinfo.h"
 #include "lqt_qtvr.h"
 
+#pragma GCC visibility push(default)
+
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -73,10 +76,10 @@ int lqt_fileno(quicktime_t *file);
  *  \param value Parameter value.
  *
  *  For integer parameters, value must be of the type int*. For string parameters,
- *  use char*.
+ *  use char*.  For floating-point parameters, use float*.
  */
   
-void lqt_set_audio_parameter(quicktime_t *file,int track, char *key,void *value);
+void lqt_set_audio_parameter(quicktime_t *file,int track, const char *key,const void *value);
 
 /** \ingroup video
  *  \brief Set a codec parameter for a video track
@@ -86,10 +89,10 @@ void lqt_set_audio_parameter(quicktime_t *file,int track, char *key,void *value)
  *  \param value Parameter value.
  *
  *  For integer parameters, value must be of the type int*. For string parameters,
- *  use char*.
+ *  use char*.  For floating-point parameters, use float*.
  */
 
-void lqt_set_video_parameter(quicktime_t *file,int track, char *key,void *value);
+void lqt_set_video_parameter(quicktime_t *file,int track, const char *key,const void *value);
 
 /** \ingroup video_decode
  *  \brief Get the pixel aspect ratio of a video track
@@ -477,6 +480,116 @@ void lqt_get_text_bg_color(quicktime_t * file, int track,
    @}
 */
 
+/** \defgroup timecodes Timecodes
+    \brief Timecode support
+
+    Timecodes are passed to/from libquicktime in the same format as they are stored in the
+    file: 32 bit unsigned integers. For the meaning of the bits, see the section "Timecode Sample Data"
+    in the Quicktime file format specification.
+    
+    @{
+
+*/
+  
+#define LQT_TIMECODE_DROP    0x0001 //!< Indicates whether the timecode is drop frame
+#define LQT_TIMECODE_24HMAX  0x0002 //!< Indicates whether the timecode wraps after 24 hours
+#define LQT_TIMECODE_NEG_OK  0x0004 //!< Indicates whether negative time values are allowed
+#define LQT_TIMECODE_COUNTER 0x0008 //!< Indicates whether the time value corresponds to a tape counter value
+
+/** \brief Attach a timecode track to a video track
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \param flags Zero or more of the LQT_TIMECODE_* flags
+ *  \param framerate The integer framerate
+ *
+ *  If the format (e.g. AVI) doesn't support timecode, this
+ *  function emits a warning.
+ *
+ */
+
+void lqt_add_timecode_track(quicktime_t * file, int track,
+                            uint32_t flags, int framerate);
+
+/** \brief Write a timecode for the next video frame to be encoded
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *
+ *  Call this function @b before encoding the actual video frame.
+ */
+  
+void lqt_write_timecode(quicktime_t * file, int track,
+                        uint32_t timecode);
+
+/** \brief Check, if a video track has timecodes
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \param flags If non NULL returns zero or more of the LQT_TIMECODE_* flags
+ *  \param framerate If non NULL returns the integer framerate
+ *  \returns 1 if timecodes are available, 0 else
+ */
+  
+int lqt_has_timecode_track(quicktime_t * file, int track,
+                           uint32_t * flags, int * framerate);
+
+/** \brief Read the timecode for the next frame to be decoded
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \param timecode Returns the timecode if available
+ *  \returns 1 if a timecode is available for this frame, 0 else
+ *
+ *  For frames, which have no timecode attached, you can increment the
+ *  last timecode accordingly in your application.
+ *  Call this function @b before decoding the actual video frame.
+ */
+
+  
+int lqt_read_timecode(quicktime_t * file, int track,
+                      uint32_t * timecode);
+
+/** \brief Get the tape name stored in a timecode track
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \returns pointer to tape name (might be NULL if no tape name)
+ *
+ *  Returned pointer should remain valid as long as the file is open.
+ *  Do not free it.
+ */
+  
+const char * lqt_get_timecode_tape_name(quicktime_t * file, int track);
+
+/** \brief Set the tapename for a timecode track
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \param tapename Tape name string
+ *
+ *  A copy is made of the string passed in
+ */
+    
+void lqt_set_timecode_tape_name(quicktime_t * file, int track,
+				const char * tapename);
+
+/** \brief Get the enabled flag of a timecode track
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \returns 1 if enabled, 0 if not enabled
+ */
+
+int lqt_get_timecode_track_enabled(quicktime_t * file, int track);
+
+/** \brief Enable or disable a timecode track
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \param enabled 0=>disable otherwise enable
+ */
+
+void lqt_set_timecode_track_enabled(quicktime_t * file, int track,
+				    int enabled);
+
+/**
+   @}
+*/
+
+  
   
 /***********************************************
  * Advanced colormodel handling.
@@ -792,7 +905,7 @@ int lqt_add_video_track(quicktime_t *file,
  *  \param stats_file Statistics file
  *
  *  This is a purely optional function, which enables multipass encoding.
- *  Multipass transcoding is done by repeatedly opening a quicktime file,
+ *  Multipass encoding is done by repeatedly opening a quicktime file,
  *  encoding video and and closing it again. The stats_file parameter must
  *  always be the same for all passes.
  *
@@ -1362,5 +1475,6 @@ void lqt_set_max_riff_size(quicktime_t * file, int size);
 }
 #endif /* __cplusplus */
 
+#pragma GCC visibility pop
   
 #endif
