@@ -77,7 +77,7 @@ int ffmpeg_num_video_codecs = -1;
     PARAM_QCOMPRESS, \
     PARAM_QBLUR, \
     PARAM_QUANTIZER_NOISE_SHAPING, \
-    PARAM_FLAG_TRELLIS_QUANT
+    PARAM_TRELLIS
 
 #define ENCODE_PARAM_VIDEO_QUANTIZER_IP \
   ENCODE_PARAM_VIDEO_QUANTIZER_I, \
@@ -177,11 +177,15 @@ int ffmpeg_num_video_codecs = -1;
 
 static lqt_parameter_info_static_t encode_parameters_mpeg4[] = {
   ENCODE_PARAM_VIDEO_FRAMETYPES_IPB,
+  PARAM_FLAG_AC_PRED_MPEG4,
   ENCODE_PARAM_VIDEO_RATECONTROL,
   ENCODE_PARAM_VIDEO_QUANTIZER_IPB,
+  PARAM_FLAG_CBP_RD,
   ENCODE_PARAM_VIDEO_ME,
   PARAM_FLAG_GMC,
   PARAM_FLAG_4MV,
+  PARAM_FLAG_MV0,
+  PARAM_FLAG_QP_RD,
   ENCODE_PARAM_VIDEO_ME_PRE,
   ENCODE_PARAM_VIDEO_QPEL,
   ENCODE_PARAM_VIDEO_MASKING,
@@ -354,7 +358,21 @@ struct CODECIDMAP codecidmap_v[] =
                    "MPG3", "div6", "div3", "DIV4", "div4", "AP41", "ap41",
                    "3IVD", (char *)0 },
       .wav_ids = { LQT_WAV_ID_NONE },
-      .compatibility_flags = LQT_FILE_AVI,
+      .compatibility_flags = LQT_FILE_AVI | LQT_FILE_AVI_ODML,
+      .do_encode = 1,
+    },
+    {
+      .id = CODEC_ID_MSMPEG4V3,
+      .index = -1,
+      .encoder = NULL,
+      .decoder = NULL,
+      .encode_parameters = encode_parameters_msmpeg4v3,
+      .decode_parameters = decode_parameters_video,
+      .short_name = "msmpeg4v3_wmp",
+      .name = TRS("FFMPEG MSMpeg 4v3 (WMP compatible)"),
+      .fourccs = { "MP43", (char *)0 },
+      .wav_ids = { LQT_WAV_ID_NONE },
+      .compatibility_flags = LQT_FILE_AVI | LQT_FILE_AVI_ODML,
       .do_encode = 1,
     },
 #if 0
@@ -371,6 +389,18 @@ struct CODECIDMAP codecidmap_v[] =
       .wav_ids = { LQT_WAV_ID_NONE },
     },
 #endif
+    {
+      .id = CODEC_ID_H263,
+      .index = -1,
+      .encoder = NULL,
+      .decoder = NULL,
+      .decode_parameters = decode_parameters_video,
+      .short_name = "h263",
+      .name = TRS("FFMPEG H263"),
+      .fourccs = { "H263", "h263", "U263", "u263", "s263", (char *)0 },
+      .wav_ids = { LQT_WAV_ID_NONE },
+      .compatibility_flags = LQT_FILE_QT_OLD | LQT_FILE_QT | LQT_FILE_MP4 | LQT_FILE_3GP,
+    },
     {
       .id = CODEC_ID_H263,
       .index = -1,
@@ -380,9 +410,9 @@ struct CODECIDMAP codecidmap_v[] =
       .decode_parameters = decode_parameters_video,
       .short_name = "h263",
       .name = TRS("FFMPEG H263"),
-      .fourccs = { "H263", "h263", "U263", "u263", (char *)0 },
+      .fourccs = { "H263", "h263", "s263", (char *)0 },
       .wav_ids = { LQT_WAV_ID_NONE },
-      .compatibility_flags = LQT_FILE_QT_OLD | LQT_FILE_QT,
+      .compatibility_flags = LQT_FILE_QT_OLD | LQT_FILE_QT | LQT_FILE_MP4 | LQT_FILE_3GP,
       .do_encode = 1,
     },
     {
@@ -454,7 +484,7 @@ struct CODECIDMAP codecidmap_v[] =
       .name = TRS("FFMPEG MJPEG"),
       .fourccs = { "MJPG", "mjpg", "JPEG", "jpeg", "dmb1", "AVDJ", (char *)0 },
       .wav_ids = { LQT_WAV_ID_NONE },
-      .compatibility_flags = LQT_FILE_AVI,
+      .compatibility_flags = LQT_FILE_AVI | LQT_FILE_AVI_ODML,
       .do_encode = 1,
     },
     {
@@ -566,6 +596,8 @@ struct CODECIDMAP codecidmap_v[] =
       .name = TRS("FFMPEG RLE"),
       .fourccs = { "rle ", (char *)0 },
       .wav_ids = { LQT_WAV_ID_NONE },
+      .compatibility_flags = LQT_FILE_QT_OLD | LQT_FILE_QT,
+      .do_encode = 1,
     },
     {
       .id = CODEC_ID_MSRLE,
@@ -605,6 +637,20 @@ struct CODECIDMAP codecidmap_v[] =
       .compatibility_flags = LQT_FILE_QT_OLD | LQT_FILE_QT,
       .do_encode = 1
     },
+    {
+      .id = CODEC_ID_DVVIDEO,
+      .index = -1,
+      .encoder = NULL,
+      .decoder = NULL,
+      .encode_parameters = encode_parameters_dvvideo,
+      .decode_parameters = decode_parameters_video,
+      .short_name = "dv_avi",
+      .name = TRS("FFMPEG DV (AVI version)"),
+      .fourccs = { "dvsd", (char *)0 },
+      .wav_ids = { LQT_WAV_ID_NONE },
+      .compatibility_flags = LQT_FILE_AVI | LQT_FILE_AVI_ODML,
+      .do_encode = 1
+    },
     {
       .id = CODEC_ID_DVVIDEO,
       .index = -1,
@@ -633,6 +679,55 @@ struct CODECIDMAP codecidmap_v[] =
       .compatibility_flags = LQT_FILE_QT_OLD | LQT_FILE_QT,
       .do_encode = 1
     },
+    /* DVCPRO HD (decoding only for now) */
+    {
+      .id = CODEC_ID_DVVIDEO,
+      .index = -1,
+      .encoder = NULL,
+      .decoder = NULL,
+      .encode_parameters = encode_parameters_dvvideo,
+      .decode_parameters = decode_parameters_video,
+      .short_name = "dvcprohd",
+      .name = TRS("FFMPEG DVCPROHD"),
+      .fourccs = { "dvhq", // DVCPRO HD 720p50
+                   "dvhp", // DVCPRO HD 720p60
+                   "dvh5", // DVCPRO HD 50i produced by FCP
+                   "dvh6", // DVCPRO HD 60i produced by FCP
+                   "dvh3", // DVCPRO HD 30p produced by FCP
+                   "AVd1", // AVID DV100
+                   (char *)0 },
+      .wav_ids = { LQT_WAV_ID_NONE },
+      .compatibility_flags = LQT_FILE_QT_OLD | LQT_FILE_QT,
+      .do_encode = 1
+    },
+    {
+      .id = CODEC_ID_FFVHUFF,
+      .index = -1,
+      .encoder = NULL,
+      .decoder = NULL,
+      .decode_parameters = decode_parameters_video,
+      .short_name = "ffvhuff",
+      .name = TRS("FFMPEG modified huffyuv lossless"),
+	    .fourccs = { "FFVH", (char *)0 },
+      .wav_ids = { LQT_WAV_ID_NONE },
+      .compatibility_flags = 0,
+      .do_encode = 1
+    },
+#if LIBAVCODEC_BUILD >= 3352576
+    {
+      .id = CODEC_ID_DNXHD,
+      .index = -1,
+      .encoder = NULL,
+      .decoder = NULL,
+      .decode_parameters = decode_parameters_video,
+      .short_name = "dnxhd",
+      .name = TRS("FFMPEG dnxhd"),
+            .fourccs = { "AVdn", (char *)0 },
+      .wav_ids = { LQT_WAV_ID_NONE },
+      .compatibility_flags = 0,
+      .do_encode = 1
+    },
+#endif
 };
 
 /* Audio */
@@ -662,7 +757,7 @@ struct CODECIDMAP codecidmap_a[] =
       .name = TRS("FFMPEG Mpeg Layer 2 Audio"),
       .fourccs = { ".mp2", ".MP2", (char *)0 },
       .wav_ids = { 0x55, LQT_WAV_ID_NONE },
-      .compatibility_flags = LQT_FILE_QT_OLD | LQT_FILE_QT | LQT_FILE_AVI,
+      .compatibility_flags = LQT_FILE_QT_OLD | LQT_FILE_QT | LQT_FILE_AVI | LQT_FILE_AVI_ODML,
       .do_encode = 1,
     },
     {
@@ -676,7 +771,7 @@ struct CODECIDMAP codecidmap_a[] =
       .name = TRS("FFMPEG AC3 Audio"),
       .fourccs = { ".ac3", ".AC3", (char *)0 },
       .wav_ids = { 0x2000, LQT_WAV_ID_NONE },
-      .compatibility_flags = LQT_FILE_AVI,
+      .compatibility_flags = LQT_FILE_AVI | LQT_FILE_AVI_ODML,
       .do_encode = 1,
     },
     {
@@ -796,7 +891,7 @@ static lqt_codec_info_static_t codec_info_ffmpeg =
 
 /* These are called from the plugin loader */
 
-extern int get_num_codecs()
+LQT_EXTERN int get_num_codecs()
 {
 	ffmpeg_map_init();
 	return ffmpeg_num_video_codecs + ffmpeg_num_audio_codecs;
@@ -863,7 +958,7 @@ static struct CODECIDMAP * find_codec(int index)
   return (struct CODECIDMAP *)0;
   }
 
-extern lqt_codec_info_static_t * get_codec_info(int index)
+LQT_EXTERN lqt_codec_info_static_t * get_codec_info(int index)
 {
 	struct CODECIDMAP * map;
         ffmpeg_map_init();
@@ -887,7 +982,7 @@ extern lqt_codec_info_static_t * get_codec_info(int index)
 */
 
 #define IFUNC(x) \
-void quicktime_init_video_codec_ffmpeg ## x(quicktime_video_map_t *vtrack) \
+static void quicktime_init_video_codec_ffmpeg ## x(quicktime_video_map_t *vtrack) \
 { \
 	int i; \
 	for(i = 0; i < ffmpeg_num_video_codecs; i++) { \
@@ -942,7 +1037,7 @@ IFUNC(40)
 #undef IFUNC
 #define MAX_VIDEO_FUNC 40
 #define IFUNC(x) \
-void quicktime_init_audio_codec_ffmpeg ## x(quicktime_audio_map_t *atrack) \
+static void quicktime_init_audio_codec_ffmpeg ## x(quicktime_audio_map_t *atrack) \
 { \
 	int i; \
 	for(i = 0; i < ffmpeg_num_audio_codecs; i++) { \
@@ -999,7 +1094,7 @@ IFUNC(40)
      
 #undef IFUNC
 
-extern lqt_init_video_codec_func_t get_video_codec(int index)
+LQT_EXTERN lqt_init_video_codec_func_t get_video_codec(int index)
 {
 	ffmpeg_map_init();
 	if(index > MAX_VIDEO_FUNC) {
@@ -1055,7 +1150,7 @@ extern lqt_init_video_codec_func_t get_video_codec(int index)
 	return (lqt_init_video_codec_func_t)0;
 }
 
-extern lqt_init_audio_codec_func_t get_audio_codec(int index)
+LQT_EXTERN lqt_init_audio_codec_func_t get_audio_codec(int index)
 {
 	ffmpeg_map_init();
 	if(index > MAX_AUDIO_FUNC) {
