@@ -31,6 +31,13 @@
 
 #define LOG_DOMAIN "ffmpeg_video"
 
+#ifdef HAVE_LIBSWSCALE
+#include SWSCALE_HEADER
+#endif
+
+// Enable interlaced encoding (experimental)
+// #define DO_INTERLACE
+
 typedef struct
   {
   AVCodecContext * avctx;
@@ -49,6 +56,9 @@ typedef struct
   /* Colormodel */
 
   int do_imgconvert;
+#ifdef HAVE_LIBSWSCALE
+  struct SwsContext *swsContext;
+#endif
     
   //  unsigned char ** tmp_buffer;
   unsigned char ** row_pointers;
@@ -127,23 +137,29 @@ static int lqt_ffmpeg_delete_video(quicktime_video_map_t *vtrack)
 	if(codec->initialized)
           {
           if(codec->avctx->stats_in)
-            free(codec->avctx->stats_in);
+            av_free(codec->avctx->stats_in);
           avcodec_close(codec->avctx);
           }
-        else
-          free(codec->avctx);
+        av_free(codec->avctx);
         
         if(codec->frame_buffer) free(codec->frame_buffer);
 	if(codec->buffer) free(codec->buffer);
         if(codec->row_pointers) free(codec->row_pointers);
 
-        if(codec->frame) free(codec->frame);
+        if(codec->frame) av_free(codec->frame);
+
+#ifdef HAVE_LIBSWSCALE
+        if(codec->swsContext)
+          sws_freeContext(codec->swsContext);
+#endif
+
         
 	free(codec);
 	
 	return 0;
 }
 
+#ifndef HAVE_LIBSWSCALE
 static void fill_avpicture(AVPicture * ret, unsigned char ** rows, int lqt_colormodel,
                            int row_span, int row_span_uv)
   {
@@ -186,6 +202,7 @@ static void fill_avpicture(AVPicture * ret, unsigned char ** rows, int lqt_color
       break;
     }
   }
+#endif
 
 static enum PixelFormat lqt_ffmpeg_get_ffmpeg_colormodel(int id)
   {
@@ -262,13 +279,19 @@ static void convert_image_decode_rgba(AVFrame * in_frame,
  *  to a libquicktime special one
  */
 
-static void convert_image_decode(AVFrame * in_frame, enum PixelFormat in_format,
+static void convert_image_decode(quicktime_ffmpeg_video_codec_t *codec,
+                                 AVFrame * in_frame, enum PixelFormat in_format,
                                  unsigned char ** out_frame, int out_format,
                                  int width, int height, int row_span, int row_span_uv)
   {
+#ifdef HAVE_LIBSWSCALE
+  uint8_t * out_planes[4];
+  int out_strides[4];
+#else
   AVPicture in_pic;
   AVPicture out_pic;
-
+#endif
+  
   /*
    *  Could someone please tell me, how people can make such a brain dead
    *  RGBA format like in ffmpeg??
@@ -278,6 +301,22 @@ static void convert_image_decode(AVFrame * in_frame, enum PixelFormat in_format,
     convert_image_decode_rgba(in_frame, out_frame, width, height);
     return;
     }
+
+#ifdef HAVE_LIBSWSCALE
+  out_planes[0] = out_frame[0];
+  out_planes[1] = out_frame[0];
+  out_planes[2] = out_frame[0];
+  out_planes[3] = (uint8_t*)0;
+
+  out_strides[0] = row_span;
+  out_strides[1] = row_span_uv;
+  out_strides[2] = row_span_uv;
+  out_strides[3] = 0;
+  
+  sws_scale(codec->swsContext,
+            in_frame->data, in_frame->linesize,
+            0, height, out_planes, out_strides);
+#else
   
   memset(&in_pic,  0, sizeof(in_pic));
   memset(&out_pic, 0, sizeof(out_pic));
@@ -293,6 +332,7 @@ static void convert_image_decode(AVFrame * in_frame, enum PixelFormat in_format,
   img_convert(&out_pic, lqt_ffmpeg_get_ffmpeg_colormodel(out_format),
               &in_pic,  in_format,
               width, height);
+#endif
   
   }
 
@@ -337,7 +377,12 @@ static int lqt_ffmpeg_decode_video(quicktime_t *file, unsigned char **row_pointe
     {
     codec->avctx->width           = width;
     codec->avctx->height          = height;
+
+#if LIBAVCODEC_VERSION_INT < ((52<<16)+(0<<8)+0)
     codec->avctx->bits_per_sample = quicktime_video_depth(file, track);
+#else
+    codec->avctx->bits_per_coded_sample = quicktime_video_depth(file, track);
+#endif
 
     /* Set extradata: It's done differently for each codec */
 
@@ -370,7 +415,12 @@ static int lqt_ffmpeg_decode_video(quicktime_t *file, unsigned char **row_pointe
         extradata_size = trak->mdia.minf.stbl.stsd.table[0].esds.decoderConfigLen;
         }
       }
-
+    else if((user_atom = quicktime_stsd_get_user_atom(trak, "glbl", &user_atom_len)))
+      {
+      extradata = user_atom + 8;
+      extradata_size = user_atom_len - 8;
+      }
+    
     if(extradata)
       {
       codec->extradata = calloc(1, extradata_size + FF_INPUT_BUFFER_PADDING_SIZE);
@@ -482,15 +532,36 @@ static int lqt_ffmpeg_decode_video(quicktime_t *file, unsigned char **row_pointe
     vtrack->stream_cmodel =
       lqt_ffmpeg_get_lqt_colormodel(codec->avctx->pix_fmt, &exact);
     if(!exact)
+      {
       codec->do_imgconvert = 1;
+      
+#ifdef HAVE_LIBSWSCALE
 
+      if(!((codec->avctx->pix_fmt == PIX_FMT_RGBA32) &&
+           (vtrack->stream_cmodel == BC_RGBA8888)))
+        {
+        codec->swsContext =
+          sws_getContext(width, height,
+                         codec->avctx->pix_fmt,
+                         width, height,
+                         lqt_ffmpeg_get_ffmpeg_colormodel(vtrack->stream_cmodel),
+                         0, (SwsFilter*)0,
+                         (SwsFilter*)0,
+                         (double*)0);
+        }
+#endif
+      }
     if(codec->decoder->id == CODEC_ID_DVVIDEO)
       {
       if(vtrack->stream_cmodel == BC_YUV420P)
         vtrack->chroma_placement = LQT_CHROMA_PLACEMENT_DVPAL;
       vtrack->interlace_mode = LQT_INTERLACE_BOTTOM_FIRST;
       }
-
+    else if(codec->decoder->id == CODEC_ID_MPEG4)
+      {
+      if(vtrack->stream_cmodel == BC_YUV420P)
+        vtrack->chroma_placement = LQT_CHROMA_PLACEMENT_MPEG2;
+      }
     if(codec->avctx->sample_aspect_ratio.num)
       {
       trak->mdia.minf.stbl.stsd.table[0].pasp.hSpacing = codec->avctx->sample_aspect_ratio.num;
@@ -530,7 +601,7 @@ static int lqt_ffmpeg_decode_video(quicktime_t *file, unsigned char **row_pointe
     }
   else
     {
-    convert_image_decode(codec->frame, codec->avctx->pix_fmt,
+    convert_image_decode(codec, codec->frame, codec->avctx->pix_fmt,
                          row_pointers, vtrack->stream_cmodel,
                          width, height, vtrack->stream_row_span,
                          vtrack->stream_row_span_uv);
@@ -612,6 +683,27 @@ static int lqt_ffmpeg_encode_video(quicktime_t *file, unsigned char **row_pointe
         if(!row_pointers)
           {
           vtrack->stream_cmodel = codec->encode_colormodel;
+
+          if(codec->encode_colormodel == BC_YUV420P)
+            {
+            if(codec->encoder->id == CODEC_ID_MPEG4)
+              {
+              vtrack->chroma_placement = LQT_CHROMA_PLACEMENT_MPEG2;
+#ifndef DO_INTERLACE // Seems not to work yet
+            /* enable interlaced encoding */
+              vtrack->interlace_mode = LQT_INTERLACE_NONE;
+#endif
+              }
+            else if(codec->encoder->id == CODEC_ID_DVVIDEO)
+              {
+              vtrack->chroma_placement = LQT_CHROMA_PLACEMENT_DVPAL;
+              }
+#ifndef DO_INTERLACE // Seems not to work yet
+            else
+              /* enable interlaced encoding */
+              vtrack->interlace_mode = LQT_INTERLACE_NONE;
+#endif
+            }
           return 0;
           }
         
@@ -655,20 +747,44 @@ static int lqt_ffmpeg_encode_video(quicktime_t *file, unsigned char **row_pointe
               strncpy(trak->strl->strh.fccHandler, "divx", 4);
               strncpy(trak->strl->strf.bh.biCompression, "DX50", 4);
               }
-#if 0 // Seems not to work yet
+#ifdef DO_INTERLACE // Seems not to work yet
             /* enable interlaced encoding */
             if(vtrack->interlace_mode != LQT_INTERLACE_NONE)
               {
               lqt_log(file, LQT_LOG_INFO, LOG_DOMAIN, "Enabling interlaced encoding");
               codec->avctx->flags |=
-                (CODEC_FLAG_INTERLACED_DCT|CODEC_FLAG_INTERLACED_ME);
+                (CODEC_FLAG_INTERLACED_DCT|CODEC_FLAG_INTERLACED_ME|CODEC_FLAG_ALT_SCAN);
               }
 #endif
             }
-          else if((codec->encoder->id == CODEC_ID_MSMPEG4V3) && (trak->strl))
+          else if((codec->encoder->id == CODEC_ID_MSMPEG4V3) && (trak->strl) &&
+                  !strncmp(trak->strl->strf.bh.biCompression, "DIV3", 4))
             {
             strncpy(trak->strl->strh.fccHandler, "div3", 4);
             }
+          else if((codec->encoder->id == CODEC_ID_H263) &&
+                  (file->file_type & (LQT_FILE_MP4|LQT_FILE_3GP)))
+            {
+            uint8_t d263_data[] =
+              { 'l', 'q', 't', ' ', /* Vendor? */
+                0, /* Decoder version */
+                0xa, /* Level */
+                0 /* Profile */ };
+            quicktime_user_atoms_add_atom(&(trak->mdia.minf.stbl.stsd.table[0].user_atoms),
+                                          "d263", d263_data,
+                                          sizeof(d263_data));
+            strncpy(trak->mdia.minf.stbl.stsd.table[0].format,
+                    "s263", 4);
+            }
+          else if(codec->encoder->id == CODEC_ID_FFVHUFF)
+            {
+            if(!(file->file_type & (LQT_FILE_AVI|LQT_FILE_AVI_ODML)))
+              {
+              codec->avctx->flags |= CODEC_FLAG_GLOBAL_HEADER;
+              codec->write_global_header = 1;
+              }
+            }
+          
           /* Initialize 2-pass */
           if(codec->total_passes)
             {
@@ -701,13 +817,11 @@ static int lqt_ffmpeg_encode_video(quicktime_t *file, unsigned char **row_pointe
           codec->buffer = malloc(codec->buffer_alloc);
           if(!codec->buffer)
             return -1;
-
+          
           if(codec->avctx->max_b_frames > 0)
             vtrack->has_b_frames = 1;
           
           codec->initialized = 1;
-
-          
           
 	}
         //        codec->lqt_colormodel = ffmepg_2_lqt(codec->com.ffcodec_enc);
@@ -721,7 +835,7 @@ static int lqt_ffmpeg_encode_video(quicktime_t *file, unsigned char **row_pointe
         codec->frame->pts = vtrack->timestamp;
         if(codec->avctx->flags & CODEC_FLAG_QSCALE)
           codec->frame->quality = codec->qscale;
-#if 0
+#ifdef DO_INTERLACE
         if(vtrack->interlace_mode != LQT_INTERLACE_NONE)
           {
           codec->frame->interlaced_frame = 1;
@@ -734,6 +848,9 @@ static int lqt_ffmpeg_encode_video(quicktime_t *file, unsigned char **row_pointe
                                              codec->buffer_alloc,
                                              codec->frame);
 
+        if(bytes_encoded < 0)
+          return -1;
+        
         if(bytes_encoded)
           {
           vtrack->coded_timestamp = codec->avctx->coded_frame->pts;
@@ -759,32 +876,43 @@ static int lqt_ffmpeg_encode_video(quicktime_t *file, unsigned char **row_pointe
           }
         
         /* Check whether to write the global header */
-        
+
         if(codec->write_global_header && !codec->global_header_written)
           {
-          esds = quicktime_set_esds(trak,
-                                    codec->avctx->extradata,
-                                    codec->avctx->extradata_size);
+          if(codec->encoder->id == CODEC_ID_FFVHUFF)
+            {
+            quicktime_user_atoms_add_atom(&(trak->mdia.minf.stbl.stsd.table[0].user_atoms),
+                                          "glbl",
+                                          codec->avctx->extradata, codec->avctx->extradata_size );
+            }
+          else if(codec->encoder->id == CODEC_ID_MPEG4)
+            {
+            esds = quicktime_set_esds(trak,
+                                      codec->avctx->extradata,
+                                      codec->avctx->extradata_size);
           
-          esds->version         = 0;
-          esds->flags           = 0;
+            esds->version         = 0;
+            esds->flags           = 0;
           
-          esds->esid            = 0;
-          esds->stream_priority = 0;
+            esds->esid            = 0;
+            esds->stream_priority = 0;
           
-          esds->objectTypeId    = 32; /* MPEG-4 video */
-          esds->streamType      = 0x11; /* from qt4l and CDR_Dinner_350k-994.mp4 */
-          esds->bufferSizeDB    = 64000; /* Hopefully not important :) */
+            esds->objectTypeId    = 32; /* MPEG-4 video */
+            esds->streamType      = 0x11; /* from qt4l and CDR_Dinner_350k-994.mp4 */
+            esds->bufferSizeDB    = 64000; /* Hopefully not important :) */
           
-          /* Maybe correct these later? */
-          esds->maxBitrate      = 200000;
-          esds->avgBitrate      = 200000;
+            /* Maybe correct these later? */
+            esds->maxBitrate      = 200000;
+            esds->avgBitrate      = 200000;
+          
+            /* Set iods profile */
+            if(codec->avctx->max_b_frames || (codec->avctx->flags & (CODEC_FLAG_QPEL|CODEC_FLAG_GMC)))
+              file->moov.iods.videoProfileId = 0xf3; // Advanced Simple Profile @ Level 3
+            else
+              file->moov.iods.videoProfileId = 0x03; // Simple Profile @ Level 3
+            }
           codec->global_header_written = 1;
-          
-          /* Set iods profile */
-          file->moov.iods.videoProfileId = 0xf3; // MPEG-4 Adv Simple@L3
           }
-
         return result;
 }
 
@@ -805,6 +933,10 @@ static int flush(quicktime_t *file, int track)
                                              codec->buffer,
                                              codec->buffer_alloc,
                                              (AVFrame*)0);
+
+        if(bytes_encoded < 0)
+          return 0;
+        
         vtrack->coded_timestamp = codec->avctx->coded_frame->pts;
 
         if(bytes_encoded)
@@ -835,8 +967,8 @@ static int flush(quicktime_t *file, int track)
 
 static int set_parameter_video(quicktime_t *file, 
                                int track, 
-                               char *key, 
-                               void *value)
+                               const char *key, 
+                               const void *value)
   {
   quicktime_ffmpeg_video_codec_t *codec = ((quicktime_codec_t*)file->vtracks[track].codec)->priv;
 #if 1
@@ -868,10 +1000,13 @@ void quicktime_init_video_codec_ffmpeg(quicktime_video_map_t *vtrack, AVCodec *e
 	if (quicktime_match_32(compressor, "dvc "))
 	   codec->encode_colormodel = BC_YUV411P;
         else if (quicktime_match_32(compressor, "dv5n") ||
-                 quicktime_match_32(compressor, "dv5p"))
+                 quicktime_match_32(compressor, "dv5p") ||
+                 quicktime_match_32(compressor, "AVdn"))
             codec->encode_colormodel = BC_YUV422P;
         else if (quicktime_match_32(compressor, "MJPG"))
           codec->encode_colormodel = BC_YUVJ420P;
+        else if (quicktime_match_32(compressor, "rle "))
+          codec->encode_colormodel = BC_RGB888;
         //        else if(quicktime_match_32(compressor, "dvcp"))
         //          codec->encode_colormodel = BC_YUV411P;
         else
