@@ -26,7 +26,8 @@
 #include <x264.h>
 #include <stdlib.h>
 #include <string.h>
-#include "colormodels.h"
+#include <quicktime/colormodels.h>
+#include "qtx264.h"
 
 #define LOG_DOMAIN "x264"
 
@@ -109,11 +110,11 @@ static void dump_params(x264_param_t * params)
   lqt_dump("    b_psnr:             %d\n", params->analyse.b_psnr);
 
   lqt_dump("  Rate control:\n");
+  lqt_dump("    i_rc_method:        %d\n", params->rc.i_rc_method);
   lqt_dump("    i_qp_constant:      %d\n", params->rc.i_qp_constant);
   lqt_dump("    i_qp_min:           %d\n", params->rc.i_qp_min);
   lqt_dump("    i_qp_max:           %d\n", params->rc.i_qp_max);
   lqt_dump("    i_qp_step:          %d\n", params->rc.i_qp_step);
-  lqt_dump("    b_cbr:              %d\n", params->rc.b_cbr);
   lqt_dump("    i_bitrate:          %d\n", params->rc.i_bitrate);
 #if X264_BUILD < 54
   lqt_dump("    i_rf_constant:      %d\n", params->rc.i_rf_constant);
@@ -464,6 +465,89 @@ static int set_pass_x264(quicktime_t *file,
   return 1;
   }
 
+static struct
+  {
+  int             x264_level;
+  lqt_log_level_t lqt_level;
+  }
+log_levels[] =
+  {
+    { X264_LOG_ERROR,   LQT_LOG_ERROR   },
+    { X264_LOG_WARNING, LQT_LOG_WARNING },
+    { X264_LOG_INFO,    LQT_LOG_INFO    },
+    { X264_LOG_DEBUG,   LQT_LOG_DEBUG   },
+    { X264_LOG_NONE,    LQT_LOG_WARNING }
+  };
+
+static void
+x264_do_log( void * priv, int i_level, const char *psz, va_list argp)
+  {
+  int i;
+  lqt_log_level_t lqt_level;
+  quicktime_t * file;
+  char * msg_string;
+  
+#ifndef HAVE_VASPRINTF
+  int len;
+#endif
+  file = (quicktime_t *)priv;
+  for(i = 0; i < sizeof(log_levels) / sizeof(log_levels[0]); i++)
+    {
+    if(log_levels[i].x264_level == i_level)
+      {
+      lqt_level = log_levels[i].lqt_level;
+      break;
+      }
+    }
+  if(i >= sizeof(log_levels) / sizeof(log_levels[0]))
+    {
+    lqt_log(file, LQT_LOG_WARNING,
+            LOG_DOMAIN, "Invalid log level from x264");
+    return;
+    }
+#ifndef HAVE_VASPRINTF
+  len = vsnprintf((char*)0, 0, psz, argp);
+  msg_string = malloc(len+1);
+  vsnprintf(msg_string, len+1, psz, argp);
+#else
+  vasprintf(&msg_string, psz, argp);
+#endif
+
+  i = strlen(msg_string);
+   
+  if(msg_string[i-1] == '\n')
+    msg_string[i-1] = '\0';
+     
+  
+  lqt_logs(file, lqt_level, LOG_DOMAIN, msg_string);
+  free(msg_string);
+  }
+
+
+static void set_fiel(quicktime_t * file, int track)
+  {
+  quicktime_video_map_t *vtrack = &(file->vtracks[track]);
+  quicktime_stsd_table_t *stsd;
+  
+  stsd = vtrack->track->mdia.minf.stbl.stsd.table;
+
+  if(stsd->has_fiel)
+    return;
+  
+  switch(vtrack->interlace_mode)
+    {
+    case LQT_INTERLACE_NONE:
+      lqt_set_fiel(file, track, 1, 0);
+      break;
+    case LQT_INTERLACE_TOP_FIRST:
+      lqt_set_fiel(file, track, 2, 9);
+      break;
+    case LQT_INTERLACE_BOTTOM_FIRST:
+      lqt_set_fiel(file, track, 2, 14);
+      break;
+    }
+  }
+
 
 static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
   {
@@ -490,6 +574,7 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 
   if(!codec->initialized)
     {
+
     codec->work_buffer_size = width * height * 3;
     codec->work_buffer = malloc(codec->work_buffer_size); /* Any smaller value here? */
     
@@ -510,6 +595,20 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
     codec->params.i_fps_num = lqt_video_time_scale(file, track);
     codec->params.i_fps_den = lqt_frame_duration(file, track, NULL);
 
+    // Set up logging
+    codec->params.pf_log = x264_do_log;
+    codec->params.p_log_private = file;
+    
+#if X264_BUILD >= 51
+    if(lqt_get_interlace_mode(file, track) != LQT_INTERLACE_NONE)
+      {
+      lqt_log(file, LQT_LOG_INFO, LOG_DOMAIN,
+              "Choosing interlaced encoding");
+      codec->params.b_interlaced = 1;
+      set_fiel(file, track);
+      }
+#endif
+    
     /* Set multipass control */
 
     if(codec->total_passes)
@@ -534,9 +633,14 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
         codec->params.rc.b_stat_read = 1;
         }
       }
+
+#ifdef DUMP_CONFIG    
+    dump_params(&codec->params);
+#endif
     
     /* Open encoder */
 
+
     codec->enc = x264_encoder_open(&codec->params);
     if(!codec->enc)
       {
@@ -544,6 +648,7 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
       return result;
       }
 
+    
     if(codec->params.i_bframe)
       vtrack->has_b_frames = 1; 
     
@@ -555,10 +660,8 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 
     file->moov.iods.videoProfileId = 0x15;
     
-#ifdef DUMP_CONFIG    
-    dump_params(&codec->params);
-#endif
     codec->initialized = 1;
+
     
     }
 
@@ -660,21 +763,33 @@ enum_t rc_methods[] =
     { "Average bitrate",  X264_RC_ABR },
     { "CRF based VBR",    X264_RC_CRF }
   };
-                                     
+
+#if X264_BUILD >= 63
+enum_t bframe_adaptives[] =
+  {
+    { "None",    X264_B_ADAPT_NONE },
+    { "Fast",    X264_B_ADAPT_FAST },
+    { "Trellis", X264_B_ADAPT_TRELLIS }
+  };
+#endif
 
 static int set_parameter(quicktime_t *file, 
                          int track, 
-                         char *key, 
-                         void *value)
+                         const char *key, 
+                         const void *value)
   {
   int found = 0, i;
   quicktime_x264_codec_t *codec = ((quicktime_codec_t*)file->vtracks[track].codec)->priv;
-
+  
   INTPARAM("x264_i_keyint_max", codec->params.i_keyint_max);
   INTPARAM("x264_i_keyint_min", codec->params.i_keyint_min);
   INTPARAM("x264_i_scenecut_threshold", codec->params.i_scenecut_threshold);
   INTPARAM("x264_i_bframe", codec->params.i_bframe);
+#if X264_BUILD < 63
   INTPARAM("x264_b_bframe_adaptive", codec->params.b_bframe_adaptive);
+#else
+  ENUMPARAM("x264_i_bframe_adaptive", codec->params.i_bframe_adaptive, bframe_adaptives);
+#endif
   INTPARAM("x264_i_bframe_bias", codec->params.i_bframe_bias);
   INTPARAM("x264_b_bframe_pyramid", codec->params.b_bframe_pyramid);
 
@@ -765,12 +880,16 @@ static int set_parameter(quicktime_t *file,
   ENUMPARAM("x264_i_me_method", codec->params.analyse.i_me_method, me_methods);
 
   INTPARAM("x264_i_subpel_refine", codec->params.analyse.i_subpel_refine);
+#if X264_BUILD < 65
   INTPARAM("x264_b_bframe_rdo", codec->params.analyse.b_bframe_rdo);
+#endif
   INTPARAM("x264_i_me_range", codec->params.analyse.i_me_range);
   INTPARAM("x264_i_frame_reference", codec->params.i_frame_reference);
   INTPARAM("x264_b_chroma_me", codec->params.analyse.b_chroma_me);
   INTPARAM("x264_b_mixed_references", codec->params.analyse.b_mixed_references);
+#if X264_BUILD < 65
   INTPARAM("x264_b_bidir_me", codec->params.analyse.b_bidir_me);
+#endif
   INTPARAM("x264_b_weighted_bipred", codec->params.analyse.b_weighted_bipred);
     
   ENUMPARAM("x264_i_direct_mv_pred", codec->params.analyse.i_direct_mv_pred, direct_modes);
