@@ -93,20 +93,9 @@ void lqt_registry_unlock()
   pthread_mutex_unlock(&codecs_mutex);
   }
 
-int lqt_registry_initialized()
-  {
-  int ret;
-  
-  lqt_registry_lock();
-  ret = registry_init_done;
-  lqt_registry_unlock();
-  
-  return ret;
-  }
-
 /* Free memory of parameter info */
 
-void destroy_parameter_info(lqt_parameter_info_t * p)
+static void destroy_parameter_info(lqt_parameter_info_t * p)
   {
   int i;
   if(p->name)
@@ -145,7 +134,7 @@ void destroy_parameter_info(lqt_parameter_info_t * p)
 
 /* Free memory of codec info (public) */
 
-void destroy_codec_info(lqt_codec_info_t * ptr)
+static void destroy_codec_info(lqt_codec_info_t * ptr)
   {
   int i;
 
@@ -334,7 +323,7 @@ copy_codec_info(const lqt_codec_info_t * info)
   if(ret->num_encoding_parameters)
     {
     ret->encoding_parameters =
-      calloc(ret->num_encoding_parameters, sizeof(lqt_parameter_info_t));
+      calloc(ret->num_encoding_parameters+1, sizeof(lqt_parameter_info_t));
 
     for(i = 0; i < ret->num_encoding_parameters; i++)
       copy_parameter_info(&(ret->encoding_parameters[i]),
@@ -721,16 +710,21 @@ void lqt_registry_init()
   lqt_codec_info_t * tmp_file_codecs;
   const char* plugin_dir = PLUGIN_DIR;
 
+  lqt_registry_lock();
+  if(registry_init_done)
+    {
+    lqt_registry_unlock();
+    return;
+    }
+
+  registry_init_done = 1;
+  
   /* Check for environment variable for plugin dir */
   if(getenv("LIBQUICKTIME_PLUGIN_DIR"))
     {
     plugin_dir = getenv("LIBQUICKTIME_PLUGIN_DIR");
     }
   
-  lqt_registry_lock();
-  
-  registry_init_done = 1;
-  
   if(lqt_audio_codecs || lqt_video_codecs)
     {
     lqt_registry_unlock();
@@ -830,48 +824,6 @@ const lqt_codec_info_t * lqt_get_video_codec_info(int index)
 
 /* Thread save methods of getting codec infos */
 
-lqt_codec_info_t * lqt_get_audio_codec_info_c(int index)
-  {
-  const lqt_codec_info_t * info;
-  lqt_codec_info_t * ret;
-
-  if (!lqt_registry_initialized()) /* also init registry */
-  	lqt_registry_init();
-	
-  lqt_registry_lock();
-
-  info = lqt_get_audio_codec_info(index);
-
-  if(info)
-    ret = copy_codec_info(info);
-  else
-    ret = (lqt_codec_info_t*)0;
-  lqt_registry_unlock();
-  return ret;
-  }
-
-lqt_codec_info_t * lqt_get_video_codec_info_c(int index)
-  {
-  const lqt_codec_info_t * info;
-  lqt_codec_info_t * ret;
-
-  if (!lqt_registry_initialized()) /* also init registry */
-  	lqt_registry_init();
-
-  lqt_registry_lock();
-
-  info = lqt_get_video_codec_info(index);
-
-  if(info)
-    ret = copy_codec_info(info);
-  else
-    ret = (lqt_codec_info_t*)0;
-  lqt_registry_unlock();
-  return ret;
-  }
-
-
-
 static void 
 create_parameter_info(lqt_parameter_info_t * ret,
                     const lqt_parameter_info_static_t * info)
@@ -1200,8 +1152,8 @@ lqt_codec_info_t ** lqt_find_audio_codec(char * fourcc, int encode)
 
   lqt_codec_info_t ** ret = (lqt_codec_info_t **)0;
 
-  if (!lqt_registry_initialized()) /* also init registry */
-    lqt_registry_init();
+  /* also init registry */
+  lqt_registry_init();
     
   lqt_registry_lock();
   
@@ -1213,8 +1165,8 @@ lqt_codec_info_t ** lqt_find_audio_codec(char * fourcc, int encode)
       {
       if(MATCH_FOURCC(ptr->fourccs[j], fourcc))
         {
-        if(((encode) & (ptr->direction != LQT_DIRECTION_DECODE)) ||
-           (ptr->direction != LQT_DIRECTION_ENCODE))
+        if((encode && (ptr->direction != LQT_DIRECTION_DECODE)) ||
+           (!encode && (ptr->direction != LQT_DIRECTION_ENCODE)))
           {
           tmp_ptr = ptr;
           break;
@@ -1242,8 +1194,8 @@ lqt_codec_info_t ** lqt_find_audio_codec_by_wav_id(int wav_id, int encode)
 
   lqt_codec_info_t ** ret = (lqt_codec_info_t **)0;
 
-  if (!lqt_registry_initialized()) /* also init registry */
-  	lqt_registry_init();
+  /* also init registry */
+  lqt_registry_init();
   
   lqt_registry_lock();
   
@@ -1255,8 +1207,8 @@ lqt_codec_info_t ** lqt_find_audio_codec_by_wav_id(int wav_id, int encode)
       {
       if(ptr->wav_ids[j] == wav_id)
         {
-        if(((encode) & (ptr->direction != LQT_DIRECTION_DECODE)) ||
-           (ptr->direction != LQT_DIRECTION_ENCODE))
+        if((encode && (ptr->direction != LQT_DIRECTION_DECODE)) ||
+           (!encode && (ptr->direction != LQT_DIRECTION_ENCODE)))
           {
           tmp_ptr = ptr;
           break;
@@ -1285,8 +1237,8 @@ lqt_codec_info_t ** lqt_find_video_codec(char * fourcc, int encode)
 
   lqt_codec_info_t ** ret = (lqt_codec_info_t **)0;
   
-  if (!lqt_registry_initialized()) /* also init registry */
-     lqt_registry_init();
+  /* also init registry */
+  lqt_registry_init();
   
   lqt_registry_lock();
   
@@ -1298,8 +1250,8 @@ lqt_codec_info_t ** lqt_find_video_codec(char * fourcc, int encode)
       {
       if(MATCH_FOURCC(ptr->fourccs[j], fourcc))
         {
-        if(((encode) & (ptr->direction != LQT_DIRECTION_DECODE)) ||
-           (ptr->direction != LQT_DIRECTION_ENCODE))
+        if((encode && (ptr->direction != LQT_DIRECTION_DECODE)) ||
+           (!encode && (ptr->direction != LQT_DIRECTION_ENCODE)))
           {
           tmp_ptr = ptr;
           break;
@@ -1331,8 +1283,8 @@ lqt_codec_info_t ** lqt_query_registry(int audio, int video,
   const lqt_codec_info_t * info;
   int num_codecs = 0, num_added = 0, i;
 
-  if (!lqt_registry_initialized()) /* also init registry */
-    lqt_registry_init();
+  /* also init registry */
+  lqt_registry_init();
 
   lqt_registry_lock();
 
@@ -1402,8 +1354,8 @@ lqt_codec_info_t ** lqt_find_audio_codec_by_name(const char * name)
   if(!name)
     return ret;
   
-  if (!lqt_registry_initialized()) /* also init registry */
-     lqt_registry_init();
+   /* also init registry */
+  lqt_registry_init();
 
   lqt_registry_lock();
 
@@ -1435,8 +1387,8 @@ lqt_codec_info_t ** lqt_find_video_codec_by_name(const char * name)
   if(!name)
     return ret;
   
-  if (!lqt_registry_initialized()) /* also init registry */
-     lqt_registry_init();
+  /* also init registry */
+  lqt_registry_init();
   
   lqt_registry_lock();
 
@@ -1507,8 +1459,8 @@ void lqt_set_default_parameter(lqt_codec_type type, int encode,
   lqt_codec_info_t * codec_info;
   lqt_parameter_info_t * parameter_info;
   
-  if (!lqt_registry_initialized()) /* also init registry */
-     lqt_registry_init();
+  /* also init registry */
+  lqt_registry_init();
 
   lqt_registry_lock();
 
